# Govfeed
# A tool to facilitate Social.coop governance by posting Loomio activity to Mastodon
# By: Social.coop
# License: GPL
# Repo: [repo URL]

from mastodon import Mastodon
import feedparser
import time

# VARIABLES

# Replace with your RSS feed URL
RSS_FEED_URL = "https://www.loomio.com/socialcoop.xml"
# Character maximum for the server
char_max = 500

# QUERY THE FEED

def query_feed():
    # Load the previously stored timestamp (if any)
    try:
        with open("last_published_timestamp.txt", "r") as f:
            last_published_timestamp_str = f.read()
            last_published_timestamp = time.strptime(last_published_timestamp_str, "%Y-%m-%d %H:%M:%S")
    except FileNotFoundError:
        last_published_timestamp = time.struct_time((0, 0, 0, 0, 0, 0, 0, 1, -1))

    # Fetch the RSS feed
    feed = feedparser.parse(RSS_FEED_URL)

    # Get the latest published timestamp from the feed's entries
    latest_published_timestamps = [entry.get("updated_parsed", entry.get("published_parsed")) for entry in feed.entries]
    latest_published_timestamp = max(latest_published_timestamps, key=lambda t: t[0])

    print("Latest timestamp: " + str(latest_published_timestamp))
    print("Last timestamp: " + str(last_published_timestamp))

    # Check if the feed has been updated
    if latest_published_timestamp[0] > time.mktime(last_published_timestamp):
        print("The RSS feed has been updated!")

        # Add entries more recent than the last_published_timestamp to an array
        updated_entries = [entry for entry in feed.entries if time.mktime(entry.get("updated_parsed", entry.get("published_parsed"))) > time.mktime(last_published_timestamp)]

        print("Entries loaded: " + str(len(updated_entries)))
        
        # Save the latest published timestamp for the next comparison
        with open("last_published_timestamp.txt", "w") as f:
            f.write(time.strftime("%Y-%m-%d %H:%M:%S", latest_published_timestamp))

        if len(updated_entries) > 0:
            # Reverse entry order for posting
            updated_entries = sorted(updated_entries, key=lambda entry: entry.updated, reverse=True)

            print("Entries sorted: " + str(len(updated_entries)))
            
        return updated_entries
    else:
        print("The feed has not been updated.")
        return False

# MASTODON POSTING

def post_feed(updated_entries):
    # Connect to Mastodon
    masto = Mastodon(client_id='clientcred.secret', access_token='usercred.secret')
    ## Get account information
    account_info = masto.account_verify_credentials()
    ## Extract and print account ID
    acc_id = account_info['id']
    print("Logged in to account: " + str(acc_id))

    print("Mastodon initialized")

    print("Entries to post: " + str(len(updated_entries)))
    
    for entry in updated_entries:
        entry_post = "New on Loomio:\n"
        entry_post += entry.title + "\n"
        entry_post += entry.link
        masto.status_post(entry_post[:char_max])
        print("Posted: " + entry_post)

# RUNTIME

query_outcome = query_feed()
if query_outcome:
    post_feed(query_outcome)
print("Done")
